module Race::Run
  class Droplets < Base
    description 'Show Droplets'
    syntax 'race droplets'
    def run
      result = barge.droplet.all
      if !result.success?
        puts 'Error: Please check your information'.red
      else
        puts 'Your Droplets'.yellow
        droplets = []

        droplets << [
          'ID',
          'Name',
          'IP Address',
          'Status',
          'Created At'
        ]

        result.droplets.each do |droplet|
          droplets << [
            droplet.id,
            droplet.name.to_s.red,
            droplet.networks.v4[0].ip_address.to_s.red,
            droplet.status == 'active' ? 'Active'.green : 'Deactive'.red,
            droplet.created_at
          ]
        end
        table = Terminal::Table.new rows: droplets
        puts table
      end
    end

    description 'Create new droplet'
    syntax 'race droplets new [DROPLET_NAME] [SIZE_ID] [IMAGE_ID] [REGION_ID]'
    def new(*args)
      name = args[0]
      size = args[1]
      image_id = args[2]
      region_id = args[3]
      if name.nil? || size.nil? || image_id.nil? || region_id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets new [DROPLET_NAME] [SIZE_ID] [IMAGE_ID] [REGION_ID]'.yellow
      else
        result = barge.droplet.create(name: name, region: region_id, size: size, image: image_id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Droplet created'.green
        end
      end
    end

    description 'Destroy droplet'
    syntax 'race droplets destroy [DROPLET_ID]'
    def destroy(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets destroy [DROPLET_ID]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.destroy(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Droplet destroyed'.green
        end
      end
    end

    description 'Reboot droplet'
    syntax 'race droplets reboot [DROPLET_ID]'
    def reboot(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets reboot [DROPLET_ID]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.reboot(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Droplet rebooted'.green
        end
      end
    end

    description 'This method allows you to power cycle a droplet. This will turn off the droplet and then turn it back on.'
    syntax 'race droplets cycle [DROPLET_ID]'
    def cycle(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets cycle [DROPLET_ID]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.power_cycle(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Power cycle has been successful'.green
        end
      end
    end

    description 'This method allows you to shutdown a running droplet. The droplet will remain in your account.'
    syntax 'race droplets down [DROPLET_ID]'
    def down(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets down [DROPLET_ID]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.shutdown(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Shut down has been successful'.green
        end
      end
    end

    description 'This method allows you to poweroff a running droplet. The droplet will remain in your account.'
    syntax 'race droplets off [DROPLET_ID]'
    def off(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets off [DROPLET_ID]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.power_off(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Power off has been successful'.green
        end
      end
    end

    description 'This method allows you to poweron a powered off droplet.'
    syntax 'race droplets on [DROPLET_ID]'
    def on(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets on [DROPLET_ID]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.power_on(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Power on has been successful'.green
        end
      end
    end

    description 'This method will reset the root password for a droplet. Please be aware that this will reboot the droplet to allow resetting the password.'
    syntax 'race droplets reset_password [DROPLET_ID]'
    def reset_password(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets reset_password [DROPLET_ID]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.password_reset(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Password restored. Please check your email'.green
        end
      end
    end

    description 'This method allows you to resize a specific droplet to a different size. This will affect the number of processors and memory allocated to the droplet.'
    syntax 'race droplets resize [DROPLET_ID] [SIZE_ID]'
    def resize(*args)
      id = args[0]
      size_id = args[1]
      if id.nil? || size_id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets resize [DROPLET_ID] [SIZE_ID]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        fail ArgumentError, "Argument Error - #{id}" unless size_id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.resize(id, size: size_id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Droplet resized'.green
        end
      end
    end

    description 'This method allows you to take a snapshot of the droplet once it has been powered off, which can later be restored or used to create a new droplet from the same image. Please be aware this may cause a reboot.'
    syntax 'race droplets create-snaphot [DROPLET_ID] [SNAPSHOT_NAME]'
    def create_snapshot(*args)
      id = args[0]
      name = args[1]
      if id.nil? || name.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets snaphot [DROPLET_ID] [SNAPSHOT_NAME]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.snapshot(id, name: name)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Snapshot generated.'.green
        end
      end
    end

    description 'Show droplet snapshots'
    syntax 'race droplets snaphots [DROPLET_ID]'
    def snapshots(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets snaphots [DROPLET_ID] '.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.snapshots(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Your Droplets'.yellow
          snapshots = []
          snapshots << [
            'Name',
            'Distribution',
            'Public',
            'Created At'
          ]
          result.snapshots.each do |snapshot|
            snapshots << [
              snapshot.name.to_s.red,
              snapshot.distribution.to_s.red,
              snapshot.status.to_s == 'active' ? 'Active'.green : 'Deactive'.red,
              snapshot.created_at
            ]
          end
          table = Terminal::Table.new rows: snapshots
          puts table
        end

      end
    end

    description 'This method allows you to restore a droplet with a previous image or snapshot. This will be a mirror copy of the image or snapshot to your droplet. Be sure you have backed up any necessary information prior to restore.'
    syntax 'race droplets restore [DROPLET_ID] [IMAGE_ID]'
    def restore(*args)
      id = args[0]
      image_id = args[1]
      if id.nil? || image_id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets restore [DROPLET_ID] [IMAGE_ID]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        fail ArgumentError, "Argument Error - #{image_id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.restore(id, image: image_id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Droplets restored.'.green
        end
      end
    end

    description 'This method allows you to reinstall a droplet with a default image. This is useful if you want to start again but retain the same IP address for your droplet.'
    syntax 'race droplets rebuild [DROPLET_ID] [IMAGE_ID]'
    def rebuild(*args)
      id = args[0]
      image_id = args[1]
      if id.nil? || image_id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets rebuild [DROPLET_ID] [IMAGE_ID]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        fail ArgumentError, "Argument Error - #{image_id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.rebuild(id, image: image_id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Droplets rebuilded.'.green
        end
      end
    end

    description 'This method renames the droplet to the specified name.'
    syntax 'race droplets rename [DROPLET_ID] [NEW_NAME]'
    def rename(*args)
      id = args[0]
      name = args[1]
      if id.nil? || name.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets rename [DROPLET_ID] [NEW_NAME]'.yellow
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.rename(id, name: name)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Droplets renamed.'.green
        end
      end
    end

    description 'Enable IPv6 for a droplet'
    syntax 'race droplets enable-ipv6 [DROPLET_ID]'
    def enable_ipv6(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets enable-ipv6 [DROPLET_ID]'
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.enable_ipv6(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'IPv6 Enabled'.green
        end
      end
    end

    description 'Show droplet kernels'
    syntax 'race droplets kernels [DROPLET_ID]'
    def kernels(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets kernels [DROPLET_ID]'
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.kernels(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Kernels'.yellow
          kernels = []

          kernels << %w(ID Name Version)

          result.kernels.each do |kernel|
            kernels << [
              kernel.id,
              kernel.name.to_s.red,
              kernel.version.to_s
            ]
          end
          table = Terminal::Table.new rows: kernels
          puts table
        end
      end
    end

    description "Change a droplet's kernel"
    syntax 'race droplets change-kernel [DROPLET_ID] [KERNEL_ID]'
    def change_kernel(*args)
      id = args[0]
      kernel_id = args[1]
      if id.nil? || kernel_id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets change-kernel [DROPLET_ID] [KERNEL_ID]'
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        fail ArgumentError, "Argument Error - #{kernel_id}" unless kernel_id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.change_kernel(id, kernel: kernel_id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Droplet kernel changed'.green
        end
      end
    end

    description 'Disable backups for a droplet'
    syntax 'race droplets disable-backup [DROPLET_ID]'
    def disable_backup(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets disable-backup [DROPLET_ID]'
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.disable_backups(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Backup disabled'.green
        end
      end
    end

    description 'Enable private networking for a droplet'
    syntax 'race droplets enable-private-network [DROPLET_ID]'
    def enable_private_network(*args)
      id = args[0]
      if id.nil?
        puts 'Argument Error'.red
        puts 'Usage'.yellow
        puts '$ race droplets enable-private-network [DROPLET_ID]'
      else
        fail ArgumentError, "Argument Error - #{id}" unless id =~ /\A[-+]?[0-9]*\.?[0-9]+\Z/
        result = barge.droplet.enable_private_networking(id)
        if !result.success?
          puts "#{result.message}".red
        else
          puts 'Backup disabled'.green
        end
      end
    end

    private

    def config(value)
      @config ||= value
    end
  end
end
